#!/bin/sh

# The bot, doing all the update and sync operations
# (c) 2008-2009, Fabien Tassin <fta@sofaraway.org>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

WORKDIR=/data/bot
BINDIR=/data/bot/src
CONFDIR=$WORKDIR/confs
SYNC_OPTS=""

#######

get_conf () {
  local conf=$1
  local key=$2
  perl -e '
    $/ = undef;
    open(F, "'$conf'") || print "Cant open conf file: $!\n";
    $s = <F>;
    close F;
    $data = eval $s;
    my @keys = split /\//, "'$key'";
    $d = $data;
    while (my $k = shift @keys) {
      exit 1 unless defined $$d{$k};
      $d = $$d{$k};
      if (ref($d) eq "ARRAY" && $#$keys == -1) {
        $d = join " ", @$d;
        last;
      }
    }
    print $d;
    exit 0;
  '
  return 0
}

send_email () {
  local dir=$1
  local pocket=$2
  local from="`echo $3 | sed -e 's/@/\\\\@/g'`"
  local to="`echo $4 | sed -e 's/@/\\\\@/g'`"
  perl -e '
    use MIME::Entity;
    my $dir = "'"$dir"'";
    my $top = MIME::Entity->build(Type    => "multipart/mixed",
                                  From    => "'"$from"'",
                                  To      => "'"$to"'",
                                  Subject => "[PPABOT] Logs for '$pocket'");

    opendir(DIR, "$dir") || die "Cant opendir $dir: $!";
    my @files = sort { -M "$dir/$b" <=> -M "$dir/$a" } readdir(DIR);
    closedir DIR;
    $top->attach(Data => "Logs for '$pocket'");
    for $file (@files) {
      next unless -f "$dir/$file";
      next if -z "$dir/$file";
      if ($file =~ m/\.gz$/) {
        $top->attach(Path     => "$dir/$file",
                     Type     => "application/x-gzip",
                     Encoding => "base64");
      }
      else {
        $top->attach(Path     => "$dir/$file");
      }
    }
    open MAIL, "| /usr/lib/sendmail -t -oi -oem" or die "Cant open sendmail: $!";
    $top->print(\*MAIL);
    close MAIL;
  '
  return 0
}

usage () {
  echo "Usage: "`basename $0`" [-n|--no-tarball-update] [-f|--force] pocket1 [pocket2] [...]"
  echo
  echo "   -e or --email                Email logs to the pocket watchers"
  echo "   -n or --no-tarball-update    Don't update tarballs"
  echo "   -s or --no-sync              Don't sync the branches (implies --no-push)"
  echo "   -p no --no-push              Prevent the final dput to the builders"
  echo "   -f or --force                Force a sync"
  echo "   -h or --help                 This help screen"
}

no_update=0
no_push=0
no_sync=0
no_email=1
while [ $# -gt 0 ]; do
  case "$1" in
    -e | --email )
      shift
      no_email=0 ;;
    -n | --no-tarball-update )
      shift
      no_update=1 ;;
    -s | --no-sync )
      shift
      no_sync=1 ;;
    -p | --no-push )
      shift
      no_push=1 ;;
    -f | --force )
      shift
      SYNC_OPTS="$SYNC_OPTS -f" ;;
    -h | --help )
      usage
      exit 0
      ;;
    -- ) # Stop option processing
      shift
      break ;;
    -* )
      echo Error: unknown option $1
      usage
      exit 1 ;;
    * )
      break ;;
  esac
done

POCKETS="$@"
if [ "Z$POCKETS" = Z ] ; then
  usage
  exit 1
fi

cd $WORKDIR

if [ $no_push -eq 0 ] ; then
  SYNC_OPTS="$SYNC_OPTS -P"
fi

# We don't want to let this dir grow
rm -rf ppa

for POCKET in $POCKETS ; do
  CONF_OPTS="-p $CONFDIR/ppabot-pkgs-$POCKET.conf -c $CONFDIR/ppabot-$POCKET.conf"
  if [ $no_email -eq 0 ] ; then
    EMAIL_DIR=`mktemp -d`
    LOG_INIT=$EMAIL_DIR/$POCKET-init.txt
    LOG_UPDATE=$EMAIL_DIR/$POCKET-update.txt
    LOG_SYNC=$EMAIL_DIR/$POCKET-sync.txt
    LOG_CLEAN=$EMAIL_DIR/$POCKET-cleanup.txt
  else
    LOG_INIT=/dev/stdout
    LOG_UPDATE=/dev/stdout
    LOG_SYNC=/dev/stdout
    LOG_CLEAN=/dev/stdout
  fi

  # Initialize, in case those are new branches
  $BINDIR/sync-ppa.pl $CONF_OPTS -i > $LOG_INIT 2>&1

  if [ $no_update -eq 0 ] ; then
    # Fetch new sources from upstream, if any
    $BINDIR/update-pkg.sh $CONF_OPTS -v > $LOG_UPDATE 2>&1
  fi

  if [ $no_sync -eq 0 ] ; then
    # Sync the branches and dput to the configured location (if -P)
    $BINDIR/sync-ppa.pl $CONF_OPTS $SYNC_OPTS > $LOG_SYNC 2>&1
  fi

  # Clean-up the tarballs directory, preserve only the freshest tarballs
  PKGS=`get_conf "$CONFDIR/ppabot-pkgs-$POCKET.conf" "order"`
  cd tarballs
  ( for pkg in `ls -1 | cut -d_ -f1 | sort | uniq` ; do
    ours=0
    for p in $PKGS ; do
      if [ $p = $pkg ] ; then
        ours=1
        break
      fi
    done
    if [ $ours -ne 1 ] ; then
      continue
    fi
    win=""
    for tarball in ${pkg}_* ; do
      v=`echo $tarball | sed -e 's/.*_\(.*\)\.orig.tar.gz/\1/'`
      if [ Z$win = Z ] ; then
        win=$v
        continue;
      fi
      RET=`dpkg --compare-versions $win gt $v && echo 0 || echo 1`
      if [ $RET -eq 1 ] ; then
        ls -1 ${pkg}_$win.orig.tar.gz
        win=$v
      else
        echo $tarball
      fi
    done
  done ) | xargs -n 1 rm -vf > $LOG_CLEAN 2>&1
  cd ..

  # Send an email to all watchers, if we have to
  if [ $no_email -eq 0 ] ; then
    FROM=`get_conf "$CONFDIR/ppabot-$POCKET.conf" "who"`
    WATCHERS=`get_conf "$CONFDIR/ppabot-$POCKET.conf" "watchers"`
    if [ "Z$WATCHERS" = Z ] ; then 
      WATCHERS="$FROM"
    fi
    for file in $EMAIL_DIR/*txt ; do
      if [ ! -s $file ] ; then
        rm $file
	continue
      fi
      if [ `stat -c %s $file` -gt 100000 ] ; then
        gzip -9 $file
      fi
    done
    send_email $EMAIL_DIR $POCKET "$FROM" "$WATCHERS"
    rm -rf $EMAIL_DIR
  fi
done
